namespace Osci.Exceptions
{
    /// <summary> Diese Exception zeigt eine der in der OSCI-Spezifikation
    /// definierten Fehlermeldungen auf Nachrichtenebene (faultcode soap:Client).
    /// 
    /// <p>Copyright © 2018 Freie Hansestadt Bremen</p>
    /// <p>Erstellt von Governikus GmbH &amp; Co. KG</p>
    /// <p>Diese Bibliothek kann von jedermann nach Maßgabe der Bremer Lizenz für
    /// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen können
    /// unter der URL www.osci.de abgerufen oder bei der
    /// Koordinierungsstelle für IT-Standards (KoSIT), Schillerstr. 1,
    /// 28195 Bremen schriftlich angefordert werden.</p>
    /// 
    /// <p>Author: P. Ricklefs, N. Büngener</p> 
    /// <p>Version: 1.8.3</p>
    /// </summary>
    public class SoapClientException
        : OsciException
    {

        /// <summary> Erzeugt ein SoapClientException-Objekt mit einem Fehlercode und erklärendem
        /// String als Message. Als Message-String wird der dem Code entsprechende Eintrag
        /// in der zum aktuellen default-Locale gehörenden Sprachdatei 
        /// (s. Package de.osci.osci12.extinterfaces.language) gesetzt.
        /// </summary>
        /// <param name="errorCode">errorCode
        /// </param>
        public SoapClientException(string errorCode) 
            : base(errorCode)
        {
        }

        /// <summary>Erzeugt ein SoapClientException-Objekt mit einem erklärenden String als
        /// Message und einem Fehlercode.
        /// </summary>
        /// <param name="faultstring">faultstring</param>
        /// <param name="oscicode">oscicode</param>
        public SoapClientException(string oscicode, string faultstring) 
            : base(faultstring, oscicode)
        {
        }
    }
}