using Osci.Cryptographic;
using Osci.Exceptions;
using Osci.Helper;

namespace Osci.Roles
{
    /// <summary> Diese Klasse stellt einen OSCI-Leser dar.
    /// 
    /// <p>Copyright © 2018 Freie Hansestadt Bremen</p>
    /// <p>Erstellt von Governikus GmbH &amp; Co. KG</p>
    /// <p>Diese Bibliothek kann von jedermann nach Maßgabe der Bremer Lizenz für
    /// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen können
    /// unter der URL www.osci.de abgerufen oder bei der
    /// Koordinierungsstelle für IT-Standards (KoSIT), Schillerstr. 1,
    /// 28195 Bremen schriftlich angefordert werden.</p>
    /// 
    /// <p>Author: P. Ricklefs, N. Büngener</p> 
    /// <p>Version: 1.8.3</p>
    /// </summary>
    public class Reader
        : Role
    {
        public X509Certificate SignatureCertificate
        {
            set
            {
                throw new UnsupportedOperationException("Readerobjekte haben kein Signaturzertifikat.");
            }
        }

        private static int _idNr = -1;

        /// <summary>  Konstruktor für den Empfang einer Nachricht. Wird als Parameter
        /// null übergeben, weil keine Verschlüsselung der Rückantwort gewünscht wird,
        /// muß trotzdem mit CipherCertificate(X509Certificate) ein
        /// Verschlüsselungszertifikat gesetzt werden, weil dies Voraussetzung für
        /// die Teilnahme an OSCI ist.
        /// </summary>
        /// <param name="decrypter">Decrypter-Objekt, welches den Inhalt der Nachricht entschlüsseln
        /// soll.
        /// </param>
        public Reader(Decrypter decrypter)
        {
            Decrypter = decrypter;
            _idNr++;
            Id += _idNr;
        }

        /// <summary> Konstruktor für das Versenden einer Nachricht.
        /// </summary>
        /// <param name="cipherCertificate">Zertifikat, mit dem die Nachricht verschlüsselt werden soll.
        /// </param>
        public Reader(X509Certificate cipherCertificate)
        {
            CipherCertificate = cipherCertificate;
            _idNr++;
            Id += _idNr;
        }
    }
}