using System.IO;
using Osci.Common;
using Osci.Extensions;
using Osci.Messagetypes;

namespace Osci.SoapHeader
{
    /// <exclude/>
    /// <summary>
    /// <p>Copyright © 2018 Freie Hansestadt Bremen</p>
    /// <p>Erstellt von Governikus GmbH &amp; Co. KG</p>
    /// <p>Diese Bibliothek kann von jedermann nach Maßgabe der Bremer Lizenz für
    /// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen können
    /// unter der URL www.osci.de abgerufen oder bei der
    /// Koordinierungsstelle für IT-Standards (KoSIT), Schillerstr. 1,
    /// 28195 Bremen schriftlich angefordert werden.</p>
    /// 
    /// <p>Author: P. Ricklefs, N. Büngener</p> 
    /// <p>Version: 1.8.3</p>
    /// </summary>
    public class DesiredLanguagesH
        : HeaderEntry
    {

        public string LanguageList
        {
            get
            {
                return _languageList;
            }

        }

        /// <summary>Liste der Sprachen im Format de,fr 
        /// </summary>
        private readonly string _languageList;

        /// <summary> Creates a new DesiredLanguageH object.
        /// </summary>
        /// <param name="languageList">
        /// </param>
        public DesiredLanguagesH(string languageList)
        {
            _languageList = languageList;
        }

        public DesiredLanguagesH(OsciMessageBuilder parentHandler, Attributes attributes)
            : this(attributes.GetValue("LanguagesList"))
        {
            parentHandler.SignatureRelevantElements.AddElement("DesiredLanguages", OsciMessageBuilder.OsciXmlns, attributes);
            RefId = attributes.GetValue("Id");
        }

        public override void WriteXml(Stream stream)
        {
            stream.Write("<" + OsciNsPrefix + ":DesiredLanguages");
            stream.Write(Ns, 0, Ns.Length);
            stream.Write(" Id=\"desiredlanguages\" LanguagesList=\"" + _languageList);
            stream.Write("\" " + SoapNsPrefix + ":actor=\"http://schemas.xmlsoap.org/soap/actor/next\" " + SoapNsPrefix + ":mustUnderstand=\"1\"></" + OsciNsPrefix + ":DesiredLanguages>");
        }
    }
}