using System.IO;
using Osci.Extensions;
using Osci.Helper;
using Osci.Roles;

namespace Osci.SoapHeader
{
    /// <exclude/>
    /// <summary>
    /// <p>Copyright © 2018 Freie Hansestadt Bremen</p>
    /// <p>Erstellt von Governikus GmbH &amp; Co. KG</p>
    /// <p>Diese Bibliothek kann von jedermann nach Maßgabe der Bremer Lizenz für
    /// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen können
    /// unter der URL www.osci.de abgerufen oder bei der
    /// Koordinierungsstelle für IT-Standards (KoSIT), Schillerstr. 1,
    /// 28195 Bremen schriftlich angefordert werden.</p>
    /// 
    /// <p>Author: N. Büngener</p> 
    /// <p>Version: 1.8.3</p>
    /// </summary>
    public class IntermediaryCertificatesH
        : CertificateH
    {

        public Intermed CipherCertificateIntermediary
        {
            get
            {
                return _cipherCertificateIntermediary;
            }
            set
            {
                _log.Debug("CipherCertificate Intermed wurde gesetzt");
                _cipherCertificateIntermediary = value;
            }
        }

        public Intermed SignatureCertificateIntermediary
        {
            get; set;
        }

        private static readonly Log _log = LogFactory.GetLog(typeof(IntermediaryCertificatesH));
        private Intermed _cipherCertificateIntermediary;

        public override void WriteXml(Stream stream)
        {
            stream.Write("<" + OsciNsPrefix + ":IntermediaryCertificates");
            stream.Write(Ns, 0, Ns.Length);
            stream.Write(" Id=\"intermediarycertificates\"  " + SoapNsPrefix + ":actor=\"http://www.w3.org/2001/12/soap-envelope/actor/none\"  " + SoapNsPrefix + ":mustUnderstand=\"1\">");

            if (_cipherCertificateIntermediary != null)
            {
                AddCipherCertificate(_cipherCertificateIntermediary, stream);
            }
            if (SignatureCertificateIntermediary != null)
            {
                AddSignatureCertificate(SignatureCertificateIntermediary, stream);
            }
            stream.Write("</" + OsciNsPrefix + ":IntermediaryCertificates>");
        }
    }
}