using System.IO;
using Osci.Extensions;

namespace Osci.SoapHeader
{
    /// <exclude/>
    /// <summary>
    /// <p>Copyright © 2018 Freie Hansestadt Bremen</p>
    /// <p>Erstellt von Governikus GmbH &amp; Co. KG</p>
    /// <p>Diese Bibliothek kann von jedermann nach Maßgabe der Bremer Lizenz für
    /// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen können
    /// unter der URL www.osci.de abgerufen oder bei der
    /// Koordinierungsstelle für IT-Standards (KoSIT), Schillerstr. 1,
    /// 28195 Bremen schriftlich angefordert werden.</p>
    /// 
    /// <p>Author: P. Ricklefs, N. Büngener</p> 
    /// <p>Version: 1.8.3</p>
    /// </summary>
    public class QualityOfTimestampH
        : HeaderEntry
    {

        public bool QualityCryptographic
        {
            get
            {
                return _quality;
            }
        }

        public bool ServiceReception
        {
            get
            {
                return _service;
            }
        }

        private readonly bool _service;
        private readonly bool _quality;

        public QualityOfTimestampH(bool serviceReception, bool qualityCryptographic)
            : this("qualityoftimestamp_" + (serviceReception ? "reception" : "creation"), serviceReception, qualityCryptographic)
        {
        }

        internal QualityOfTimestampH(string refdId, bool serviceReception, bool qualityCryptographic)
        {
            RefId = refdId;
            _service = serviceReception;
            _quality = qualityCryptographic;
        }

        public override void WriteXml(Stream stream)
        {
            stream.Write("<" + OsciNsPrefix + ":QualityOfTimestamp");
            stream.Write(Ns, 0, Ns.Length);
            stream.Write(" Id=\"" + RefId + "\" Quality=\"" +
                        (_quality ? "cryptographic" : "plain") + "\" Service=\"" +
                        (_service ? "reception" : "creation") +
                        "\" " + SoapNsPrefix + ":actor=\"http://schemas.xmlsoap.org/soap/actor/next\" " + SoapNsPrefix + ":mustUnderstand=\"1\"" +
                        "></" + OsciNsPrefix + ":QualityOfTimestamp>");
        }
    }
}